<?php
/* Copyright (C) 2007-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/shippingscanner/admin/setup.php
 *		\ingroup    shippingscanner
 *		\brief      Page to setup shippingscanner module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/lib/admin.lib.php';
dol_include_once('/shippingscanner/lib/shippingscanner.lib.php');

$langs->load("admin");
$langs->load("shippingscanner@shippingscanner");
$langs->load("opendsi@shippingscanner");

if (!$user->admin) accessforbidden();

$action = GETPOST('action','alpha');


/*
 *	Actions
 */

if (preg_match('/set_(.*)/',$action,$reg))
{
    $code=$reg[1];
    $value=(GETPOST($code) ? GETPOST($code) : 1);
    if (dolibarr_set_const($db, $code, $value, 'chaine', 0, '', $conf->entity) > 0)
    {
        Header("Location: ".$_SERVER["PHP_SELF"]);
        exit;
    }
    else
    {
        dol_print_error($db);
    }
} elseif (preg_match('/del_(.*)/',$action,$reg)) {
    $code=$reg[1];
    if (dolibarr_del_const($db, $code, $conf->entity) > 0)
    {
        Header("Location: ".$_SERVER["PHP_SELF"]);
        exit;
    }
    else
    {
        dol_print_error($db);
    }
} elseif ($action == 'set') {
    $char = GETPOST('SHIPPINGSCANNER_BATCH_NEW_LINE_ASCII_CODE', 'int');
    if ($char) {
        dolibarr_set_const($db, 'SHIPPINGSCANNER_BATCH_NEW_LINE_ASCII_CODE', $char, 'chaine', 0, '', $conf->entity);
    }
}


/*
 *	View
 */


llxHeader();

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("ShippingScannerSetup"),$linkback,'title_setup');
print "<br>\n";

$form = new Form($db);

$head=shippingscanner_prepare_head();

dol_fiche_head($head, 'settings', $langs->trans("Parameters"), 0, 'action');


print '<br>';
print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.newToken().'">';
print '<input type="hidden" name="action" value="set">';

$var=true;

print '<table class="noborder centpercent">';
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("Parameters").'</td>'."\n";
print '<td align="center">&nbsp;</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// SHIPPINGSCANNER_BATCH_NEW_LINE_ASCII_CODE
print '<tr class="oddeven">'."\n";
print '<td>'.$langs->trans("ShippingScannerSplitChar").'</td>'."\n";
print '<td align="center">&nbsp;</td>' . "\n";
print '<td class="right">'."\n";
$arr_to_select_char = array(
    '-1' => $langs->trans('AsciiKeyNone'),
	'13' => $langs->trans('AsciiKeyEnter'),
	'59' => $langs->trans('AsciiKeySemiColon'),
);
print $form->selectarray("SHIPPINGSCANNER_BATCH_NEW_LINE_ASCII_CODE", $arr_to_select_char, $conf->global->SHIPPINGSCANNER_BATCH_NEW_LINE_ASCII_CODE);
print '</td></tr>'."\n";


// SHIPPINGSCANNER_FILL_EXPEDTION_LINES
print '<tr class="oddeven">'."\n";
print '<td>'.$langs->trans("ShippingScannerFillExpeditionLinesOption").'</td>'."\n";
print '<td align="center">&nbsp;</td>' . "\n";
print '<td class="right">'."\n";
if (empty($conf->global->SHIPPINGSCANNER_FILL_EXPEDTION_LINES)) {
    print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_SHIPPINGSCANNER_FILL_EXPEDTION_LINES&token='.newToken().'">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
} else {
    print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_SHIPPINGSCANNER_FILL_EXPEDTION_LINES&token='.newToken().'">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
}
print '</td></tr>'."\n";
print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Save").'">';
print '</div>';

print '<br>';

$var=true;


dol_fiche_end();

print '<br>';
//print '<div align="center">';
//print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
//print '</div>';

print '</form>';

llxFooter();

$db->close();
